*:*********************************************************************
*:
*:       Function: CutNPaste()
*:
*:         System: GRUMPFISH LIBRARY
*:         Author: Greg Lief
*:      Copyright (c) 1988-90, Greg Lief
*:  Last modified:
*:
*:  Allows user to cut-and-paste from underlying screen into the
*:  Grumpfish notepad (POPNOTE)
*:
*:  Syntax: CutNPaste(@<screen> [, <hicolor>, <locolor>] ;
*:                              [, <top>, <left>, <bottom>, <right> ;
*:                              [, <row>, <column>]])
*:
*:  Parameters: <screen> is a character string representing the name
*:              of the previously saved underlying screen from which
*:              to cut and paste.  For performance considerations,
*:              you must pass this by reference (i.e., do not surround
*:              it with quotes and DO precede it with "@")
*:
*:              Optional parameter <hicolor> is a numeric representing
*:              which color to use for highlighting.  The default is
*:              112 (inverse, or black on white).
*:
*:              Optional parameter <locolor> is a numeric representing
*:              which color to use for the underlying screen.  The
*:              default is 7 (white on black).
*:
*:              Optional parameters <top>, <left>, <bottom>, and <right>
*:              are numerics which delimit the screen area that can be
*:              accessed.
*:
*:              Optional parameters <row> and <column> are numerics
*:              indicating initial placement of the cursor on the screen.
*:              NOTE: if these two parameters are passed, highlighting will
*:              begin IMMEDIATELY (rather than the user having to position
*:              the cursor).
*:
*:  Methodology notes: this routine is used either to cut-and-paste from
*:                     an original underlying screen, or to highlight a
*:                     section on the current screen.  Therefore, you will
*:                     either pass the screen variable, or the four
*:                     coordinates, but not both!!!!  (i.e., if you pass
*:                     the coordinates, just pass a null string as the
*:                     screen -- example: CutNPaste('', 0, 0, 24, 79)
*:
*:  Returns: A character string suitable for KEYBOARDing into a memo field
*:
*:          Calls: GFATTR()     (Assembler function in GRUMPATT.ASM)
*:
*:*********************************************************************
FUNCTION CutNPaste
PARAMETER origscrn, hicolor, locolor, mtop, mleft, mbottom, mright, mrow, mcol
PRIVATE oldscrn, t_anchor, l_anchor, b_anchor, r_anchor, mrow, mcol, key, ;
        xx, yy, shading, cutbuff, row_len, buffer

*** establish colors if not passed as parameters
hicolor = IF(TYPE('hicolor') != 'N', 112, hicolor)
locolor = IF(TYPE('locolor') != 'N', 7, locolor)

** establish box coordinates if not passed as parameters
IF PCOUNT() < 4
   mtop = 0
   mleft = 0
   mbottom = 24
   mright = 79
ENDIF
*** establish initial screen location, anchor variables, and whether or not
*** to begin highlighting immediately
IF PCOUNT() < 8
   shading = .F.   && flag to indicate whether we are highlighting or not
   STORE mtop TO t_anchor, b_anchor, mrow
   STORE mleft TO l_anchor, r_anchor, mcol
ELSE
   STORE mrow TO t_anchor, b_anchor
   STORE mcol TO l_anchor, r_anchor
   shading = .T.   && start highlighting immediately
ENDIF
oldscrn = SAVESCREEN(mtop, mleft, mbottom, mright)

*** if the original screen parameter was passed, user restore the original underlying screen
*** and then make it all the same color (white on black)
IF PCOUNT() = 1
   RESTSCREEN(0, 0, 24, 79, origscrn)
   GFAttr(0, 0, 24, 79, locolor)
ENDIF

key = 0
cutbuff = []    && this will hold the buffer to be returned back
DO WHILE .T.
   IF shading
      GFAttr(t_anchor, l_anchor, b_anchor, r_anchor, hicolor)
   ENDIF
   @ mrow, mcol SAY ''
   key = inkey(0)
   do CASE
      CASE key = 24 .and. mrow < mbottom   && who's goin' down?
         IF shading
            IF mrow < b_anchor
               GFAttr(mrow, l_anchor, mrow, r_anchor, locolor)
               t_anchor = mrow + 1
            ELSE
               b_anchor = mrow + 1
            ENDIF
         ENDIF
         mrow = mrow + 1

      CASE key = 5 .AND. mrow > mtop && up up up
         IF shading
            IF mrow > t_anchor
               GFAttr(mrow, l_anchor, mrow, r_anchor, locolor)
               b_anchor = mrow - 1
            ELSE
               t_anchor = mrow - 1
            ENDIF
         ENDIF
         mrow = mrow - 1

      CASE key = 4 .AND. mcol < mright   && right (yer bloody well right)
         IF shading
            IF mcol < r_anchor
               GFAttr(t_anchor, mcol, b_anchor, mcol, locolor)
               l_anchor = mcol + 1
            ELSE
               r_anchor = mcol + 1
            ENDIF
         ENDIF
         mcol = mcol + 1

      CASE key = 19 .AND. mcol > mleft    && left (out in the cold)
         IF shading
            IF mcol > l_anchor
               GFAttr(t_anchor, mcol, b_anchor, mcol, locolor)
               r_anchor = mcol - 1
            ELSE
               l_anchor = mcol - 1
            ENDIF
         ENDIF
         mcol = mcol - 1

      CASE key = 13  && Enter or Backspace
         IF ! shading
            shading = .T.
            STORE mrow TO t_anchor, b_anchor
            STORE mcol TO l_anchor, r_anchor
         ELSE
            EXIT
         ENDIF

      CASE key = 27   && (there is no) Escape
         EXIT

   ENDCASE
ENDDO
IF LASTKEY() = 13
   *** now to parse out all the color attributes
   buffer = savescreen(t_anchor, l_anchor, b_anchor, r_anchor)
   *** determine width of one row so that we can stick in CR/LF
   row_len = (r_anchor - l_anchor + 1) * 2
   FOR xx = 1 TO (b_anchor - t_anchor + 1)
      FOR yy = 1 TO row_len STEP 2
         cutbuff = cutbuff + substr(buffer, (xx - 1) * row_len + yy, 1)
      NEXT
      cutbuff = cutbuff + chr(13) + chr(10)
   NEXT
ENDIF
RESTSCREEN(mtop, mleft, mbottom, mright, oldscrn)
RETURN cutbuff

* end function CutNPaste()
*--------------------------------------------------------------------*

*eof $cutpast.prg
