*:*********************************************************************
*:
*:       Function: IsConfig()
*:         Author: Greg Lief - inspired by Eric Lewis
*:      Copyright (c) 1990 - All Rights Reserved
*:
*:      Checks either CONFIG.SYS or SHELL.CFG file for either
*:      FILES / FILE HANDLES and/or BUFFERS / CACHE BUFFERS statements
*:
*:      Syntax: IsConfig(<filename>, [<files>, <buffers>])
*:
*:      Required Parameter:   <filename> is the file to check.  This
*:                            should be either CONFIG.SYS or SHELL.CFG.
*:                            If you pass something other than these,
*:                            the function will default to CONFIG.SYS.
*:
*:      Optional Parameters:  <files> is a numeric representing the
*:                            required number of file handles.  If you
*:                            do not pass this, the default will be 50.
*:
*:                            <buffers> is a numeric representing the
*:                            required number of buffers.  If you do not
*:                            pass this, the default will be 10.
*:
*:      Return Value:  A logical True (.T.) if CONFIG.SYS/SHELL.CFG did
*:                     not require changes; False (.F.) if it did.
*:
*:      Examples:      *** need 90 files and 20 buffers
*:                     IF ! IsConfig("CONFIG.SYS", 90, 20)
*:                        QUIT
*:                     ENDIF
*:
*:          Calls: GFReadLine()   (function contained in $READLIN.PRG)
*:               : Center()       (function contained in $CENTER.PRG)
*:               : Yes_No()       (function contained in $YESNO.PRG)
*:               : WaitOn()       (function contained in $WAITING.PRG)
*:               : WaitOff()      (function contained in $WAITING.PRG)
*:               : ShadowBox()    (function contained in $SHADOWB.PRG)
*:               : ByeByeBox()    (function contained in $BYEBYE.PRG)
*:
*:*********************************************************************
FUNCTION IsConfig
PARAMETER mfile, mfiles, mbuffers
PRIVATE drive, no_files, no_buffers, ret_val, string, inhandle, ;
        outhandle, buffer, crlf, filestring, buffstring
mfile = IF(PCOUNT() = 0, "CONFIG.SYS", IF(! UPPER(mfile) $ "CONFIG.SYSSHELL.CFG" ;
        .OR. EMPTY(mfile), "CONFIG.SYS", UPPER(mfile)))
mfiles = IF(pcount() < 2, 50, mfiles)
mbuffers = IF(pcount() < 3, 10, mbuffers)


crlf = CHR(13) + CHR(10)
no_files   = .T.
no_buffers = .T.
ret_val  = .T.
string     = " "
filestring = "FILE" + IF(mfile = "CONFIG.SYS", "S", " HANDLES")
buffstring = IF(mfile = "SHELL.CFG", "CACHE ", "") + "BUFFERS"

** Determine which drive is the booting drive by looking at the COMSPEC.
drive = SUBSTR(GETE("COMSPEC"), 1, AT(":", GETE("COMSPEC")) - 1)
IF FILE(drive + ":\" + mfile)
   
   ** Read in from the CONFIG.SYS file out to the TEMP.SYS file
   inhandle = FOPEN(drive + ":\" + mfile, 2)
   outhandle = FCREATE(drive + ":\TEMP.CFG", 0)
   
   DO WHILE gfreadline(@string, inhandle)
      
      ** Check for the FILES and/or BUFFERS lines and the correct number
      DO CASE
         CASE filestring $ UPPER(string)
            ** check not only for adequate # of files, but that this
            ** line begins with the word FILES or FILE HANDLES
            IF upper(string) != filestring .or. ;
                   VAL(SUBSTR(string, AT("=", string) + 1)) < mfiles
               ** Write a new "FILE HANDLES=" line
               FWRITE(outhandle, filestring + "=" + LTRIM(STR(mfiles)) + crlf)
               ret_val = .F.
            ELSE
               FWRITE(outhandle, string + crlf)
            ENDIF
            no_files = .F.

         CASE buffstring $ UPPER(string)
            ** check not only for adequate # of buffers, but that this
            ** line begins with the word BUFFERS or CACHE BUFFERS
            IF upper(string) != buffstring .or. ;
                  VAL(SUBSTR(string, AT("=", string) + 1)) < mbuffers
               ** check to see if they use the Expanded Memory switch ("/X")
               buffer = "/X" $ UPPER(string)
               ** Write a new "BUFFERS=" line
               FWRITE(outhandle, buffstring + "=" + LTRIM(STR(mbuffers)) + ;
                      IF(buffer, " /X", "") + crlf)
               ret_val = .F.
            ELSE
               FWRITE(outhandle, string + crlf)
            ENDIF
            no_buffers = .F.

         OTHERWISE
            ** Write the string out to the new file
            IF string != CHR(26)
               FWRITE(outhandle, string + crlf)
            ENDIF
      ENDCASE
   ENDDO
   
   **
   IF no_files .OR. no_buffers
      ** Move to the end of the file less two bytes
      ** If the file ends with a CR/LF, erase them
      FSEEK(outhandle, -2, 2)
      ** If the file does not end with a CR/LF, skip to the very end
      string = FREADSTR(outhandle, 2)
      IF string != crlf
         FSEEK(outhandle, 2, 1)
      ENDIF
     
      ** Check to see if a FILE HANDLES= line was found
      IF no_files
         ** Write a new "FILE HANDLES=" line
         FWRITE(outhandle, filestring + "=" + LTRIM(STR(mfiles)) + crlf)
         ret_val = .F.
      ENDIF
      
      ** Check to see if a BUFFERS= line was found
      IF no_buffers
         ** Write a new CACHE BUFFERS= line
         FWRITE(outhandle, buffstring + "=" + LTRIM(STR(mbuffers)) + crlf)
         ret_val = .F.
      ENDIF
   ENDIF
   
   ** Close the input file
   FCLOSE(inhandle)
   
   ** Check to see if the file needs an EOF
   FSEEK(outhandle, 0, 2)
   IF FREADSTR(outhandle, 1) != CHR(26)
      FWRITE(outhandle, CHR(26))
   ENDIF
   
   ** Close the output file
   FCLOSE(outhandle)
   
   ** Confirm copy
   IF ! ret_val
      IF Yes_No("The " + mfile + " requires changes in order to run this program", ;
                 "Would you like me to make these changes now?")
         COPY FILE (drive + ":\" + mfile) TO (drive + ":\" + ;
                   SUBSTR(mfile, 1, AT(".", mfile) - 1) + ".bak")
         COPY FILE (drive + ":\temp.cfg")   TO (drive + ":\" + mfile)
      ELSE
         ret_val = .T.
         WaitOn("The " + mfile + " file MUST contain '" + filestring + "=" + ;
                LTRIM(STR(mfiles)) + "' and '" + ;
                buffstring + " = " + LTRIM(STR(mbuffers)) + "'")
         INKEY(5)
         WaitOff()
      ENDIF
      ERASE (drive + ":\temp.cfg")
   ENDIF
ELSE
   ** The file does not exist, so create one
   outhandle = FCREATE(drive + ":\" + mfile, 0)
   FWRITE(outhandle, filestring + "=" + LTRIM(STR(mfiles)) + crlf + ;
                     buffstring + "=" + LTRIM(STR(mbuffers)) + crlf)
   ret_val = .F.
   
   ** Check to see if the file needs an EOF
   FSEEK(outhandle, 0, 2)
   IF FREADSTR(outhandle, 1) != CHR(26)
      FWRITE(outhandle, CHR(26))
   ENDIF
   
   ** Close the output file
   FCLOSE(outhandle)
ENDIF

IF ! ret_val
   buffer = ShadowBox(10, 0, 14, 79, 2)
   center(11, "Changes have been to the " + mfile + " file." )
   center(11, "Please restart the computer by turning it off")
   center(12, "or pressing Ctrl-Alt-Del at the same time.")
   center(13, "Then start this program again.")
   INKEY(0)
   ByeByeBox(buffer)
ENDIF
RETURN (ret_val)

* end function IsConfig()
*--------------------------------------------------------------------*

* eof _isconfi.prg
